package com.softslate.commerce.businessobjects.core;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.order.CartProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.ActionUtils;

/**
 * A class which may be run from the command line to check for orders whose
 * payments were processed (through Authorize.net DPM) but where the order
 * itself was not processed and marked complete in the system. This command is
 * useful in stores using the Authorize.net DPM processor with the Complete
 * Order When? setting set to "After Payment Process". In such stores it is
 * unlikely but possible for the payment to be processed but the order not
 * completed, if some error occurs between the time Authorize.net receives the
 * response from the system and the customer's browser redirects to the Thank
 * You page. (To avoid this possibility, set the Complete Order When? setting
 * set to "During Payment Process", which is the default. Be aware processing
 * the order during processing increases the response time from the system to
 * Authorize.net, and Authorize.net may timeout and display an error to user if
 * that happens.)
 * 
 * See the accompanying <code>commandLinePaymentCheck.sh</code> file for a Unix
 * shell script that may be used to fire off the example.
 * 
 * @author dtobey
 * 
 */
public class CommandLineFailedPaymentCheck extends CommandLineSupport {

	static Log log = LogFactory.getLog(CommandLineFailedPaymentCheck.class);

	/**
	 * Main method, which gets invoked when executing this class from the
	 * command line.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		// Log the start of the process
		if (log.isInfoEnabled())
			log.info("Starting command line Failed Payment Check.");

		if (args == null || args.length > 2) {
			if (log.isErrorEnabled())
				log.error("Wrong number of arguments. 0, 1, or 2 arguments needed.");
			System.out
					.println("Usage: ./commandLinePaymentCheck.sh [olderThanDays [olderThanHours]]");
		}

		// Initilize the system objects
		Map systemObjects = initializeSystem("Command Line Admin User");
		if (systemObjects == null) {
			if (log.isErrorEnabled())
				log.error("Initialization failed ... exiting.");
			return;
		}

		// Unpack the system objects needed by this process
		BusinessObjectFactory bof = (BusinessObjectFactory) systemObjects
				.get("bof");
		// Properties appSettings = (Properties)
		// systemObjects.get("appSettings");
		// Properties appComponents = (Properties) systemObjects
		// .get("appComponents");
		// DAOFactory daoFactory = (DAOFactory) systemObjects.get("daoFactory");
		Settings settings = (Settings) systemObjects.get("settings");
		User user = (User) systemObjects.get("user");

		// Perform the processing
		if (log.isInfoEnabled())
			log.info("Doing command line Failed Payments Check.");

		try {

			OrderProcessor orderProcessor =  bof
					.createObject(OrderProcessor.class);
			CartProcessor cp =  bof
					.createObject(CartProcessor.class);

			// Default the last modified check to 1 hour, overridable by
			// arguments
			Map parameters = new HashMap();
			parameters.put("olderThanDays", "0");
			parameters.put("olderThanHours", "1");
			if (args != null && args.length > 0) {
				parameters.put("olderThanDays", args[0]);
			}
			if (args != null && args.length > 1) {
				parameters.put("olderThanHours", args[1]);
			}

			// Load the problem orders
			Collection inCompleteOrders = orderProcessor
					.loadIncompleteOrdersWithPayments(parameters);
			if (log.isInfoEnabled())
				log.info("Outputting Failed Payments check "
						+ "processing results. There are - "
						+ inCompleteOrders.size()
						+ " incomplete orders with payment records.");

			// Loop through them and perform the processing
			if (!inCompleteOrders.isEmpty() && inCompleteOrders != null) {
				StringBuffer notificationEmail = new StringBuffer(
						"There was a problem with the following order(s)."
								+ " The customer's payment was processed but something happened and the order was not marked "
								+ "complete in the system. The cron task that sent this email to you has just completed the "
								+ "order processing now. However, it did not send the order confirmation email out to the "
								+ "customer, nor to the store administrators. You may trigger those emails to be sent out by "
								+ "submitting the Send Order Emails form in the admin area for each order.\n\n");
				Iterator i = inCompleteOrders.iterator();
				while (i.hasNext()) {
					Order or = (Order) i.next();
					if (log.isInfoEnabled())
						log.info("Fixing Failed Payment on order number - "
								+ or.getOrderID() + " - " + or.getCustomerID());
					user.setOrder(or);
					try {
						cp.processOrderComplete(null);
						cp.markOrderComplete(null);
						notificationEmail.append("Order Number: ");
						notificationEmail.append(or.getOrderNumber());
						notificationEmail.append("Send email form: ");
						notificationEmail.append(settings
								.getValue("administratorURL"));
						notificationEmail.append("/SendEmailsForm.do?orderID=");
						notificationEmail.append(or.getOrderID());
						notificationEmail.append("\n");
					} catch (Exception e) {
						if (log.isErrorEnabled())
							log.error(
									"Error processing orderID: "
											+ or.getOrderID(), e);
						notificationEmail.append("Error processing Order ID: "
								+ e.toString()
								+ " (view logs for full stacktrace).");
					}
				}
				notificationEmail
						.append("\nThis email was sent from CommandLineFailedPaymentCheck.");
				sendNotificationEmail(notificationEmail.toString(), settings,
						bof);
			}
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Command line processing failed ... exiting.", e);
			return;
		}

		// Log the end of process
		if (log.isInfoEnabled())
			log.info("Ending check for Failed Payments.");
	}

	public static void sendNotificationEmail(String textMessage,
			Settings settings, BusinessObjectFactory bof) {
		try {
			ActionUtils actionUtils = (ActionUtils) bof
					.createUtilsObject("actionUtilsImplementer");
			String toEmails = (String) settings.getValue("notificationEmailTO");
			String ccEmails = (String) settings.getValue("notificationEmailCC");
			String bccEmails = (String) settings
					.getValue("notificationEmailBCC");
			String fromEmail = (String) settings.getValue("storeEmailAddress");
			String smtpServer = (String) settings.getValue("storeSMTPServer");
			String smtpUsername = (String) settings
					.getValue("storeSMTPUsername");
			String smtpPassword = (String) settings
					.getValue("storeSMTPPassword");
			String storeSMTPPort = (String) settings.getValue("storeSMTPPort");
			String characterEncoding = (String) settings
					.getValue("characterEncoding");
			String subject = "Clean Up Performed on Orders that Failed to Process";
			// Send the invoice email
			if (log.isInfoEnabled())
				log.info("Attempting to send failed payments notification email to: "
						+ toEmails);
			actionUtils.sendMultipartEmail(toEmails, ccEmails, bccEmails,
					fromEmail, smtpServer, subject, textMessage, textMessage,
					smtpUsername, smtpPassword, storeSMTPPort,
					characterEncoding);
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Error sending failed payments notification email:",
						e);
		}
	}

}
