package com.softslate.commerce.businessobjects.product;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.product.CategoryDAO;
import com.softslate.commerce.daos.product.CategoryGatewayDAO;
import com.softslate.commerce.daos.product.ProductGatewayDAO;

/**
 * Processes business logic having to do with categories. Created in the Struts
 * layer in response to requests for viewing, editing, adding, and deleting
 * categories.
 * <p>
 * <code>BasicCategoryProcessor</code> is the default implementation of the
 * <code>CategoryProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicCategoryProcessor extends BaseBusinessProcessor implements
		CategoryProcessor {

	static Log log = LogFactory.getLog(BasicCategoryProcessor.class);

	public Collection loadAllCategories() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading all categories");
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadAllCategories();
	}

	public Map loadCategoriesAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading categories and count");
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadCategoriesAndCount(parameters);
	}

	public Map addCategory(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding category: " + code);
		}
		Map results = new HashMap();

		// Check that another category with the same code doesn't exist
		Category category = (Category) getBusinessObjectFactory().createObject(
				"categoryImplementer");
		BeanUtils.copyProperties(category, parameters);
		results = validateCategory(category);
		if (results != null && results.get("resultCode") != null
				&& !results.get("resultCode").equals("0")) {
			return results;
		}

		// Insert the category
		if (category.getParentCategory() != null
				&& category.getParentCategory().intValue() != 0) {
			Category parent =  getBusinessObjectFactory()
					.createObject(Category.class);
			parent.setCategoryID(category.getParentCategory().intValue());
			category.setParent(parent);
		}
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		dao.setCategory(category);
		dao.insertCategory(true);
		results.put("newRecord", dao.getCategory());
		return results;
	}

	public Category loadCategoryFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading category from id");
		Category category = (Category) getBusinessObjectFactory().createObject(
				"categoryImplementer");
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		BeanUtils.copyProperties(category, parameters);
		dao.setCategory(category);
		dao.loadCategoryFromID();
		if (dao.getCategory() != null) {
			dao.loadParents();
		}
		return dao.getCategory();
	}

	public Category loadCategoryFromCode(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading category from code");
		Category category = (Category) getBusinessObjectFactory().createObject(
				"categoryImplementer");
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		BeanUtils.copyProperties(category, parameters);
		dao.setCategory(category);
		dao.loadCategoryFromCode();
		return dao.getCategory();
	}

	public Map editCategory(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing category: " + code);
		}
		Map results = new HashMap();

		// Check that another category with the same code doesn't exist
		Category category = (Category) getBusinessObjectFactory().createObject(
				"categoryImplementer");
		BeanUtils.copyProperties(category, parameters);
		results = validateCategory(category);
		if (results != null && results.get("resultCode") != null
				&& !results.get("resultCode").equals("0")) {
			return results;
		}

		// Load the category
		category = null;
		if (parameters.get("categoryID") != null) {
			category = loadCategoryFromID(parameters);
		}
		if (category == null && parameters.get("code") != null) {
			category = loadCategoryFromCode(parameters);
		}

		// Update the category
		BeanUtils.copyProperties(category, parameters);
		if (category.getParentCategory() != null
				&& category.getParentCategory().intValue() != 0) {
			Category parent =  getBusinessObjectFactory()
					.createObject(Category.class);
			parent.setCategoryID(category.getParentCategory().intValue());
			category.setParent(parent);
		}
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		dao.setCategory(category);
		dao.updateCategory();
		results.put("updatedRecord", dao.getCategory());
		return results;
	}

	public Map editDeleteCategories(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing/deleting multiple categories");
		String[] idValues = (String[]) parameters.get("idValues");
		String[] displayFields = (String[]) parameters.get("displayFields");
		Category[] parent = new Category[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			parent[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("parentCategory")
						|| displayField.equals("parent")) {
					displayFields[j] = "parent";
					String[] fieldValues = (String[]) parameters
							.get("parentCategory");
					if (fieldValues != null && fieldValues[i] != null
							&& !fieldValues[i].equals("")
							&& !fieldValues[i].equals("0")) {
						if (log.isDebugEnabled())
							log.debug("Creating parent category of "
									+ fieldValues[i] + " for id: "
									+ idValues[i]);
						Category object =  getBusinessObjectFactory()
								.createObject(Category.class);
						object.setCategoryID((new Integer(fieldValues[i]))
								.intValue());
						parent[i] = object;
					}
				}
			}
		}
		parameters.put("parent", parent);
		CategoryGatewayDAO dao =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		Map results = dao.editDeleteCategories(parameters, getBusinessObjectFactory());
		String keepOrdersInSync = (String) getSettings().getValue(
				"keepOrdersInSync");
		if (keepOrdersInSync != null && keepOrdersInSync.equals("1")) {
			ProductGatewayDAO gatewayDao =  getDaoFactory()
					.createDAO(ProductGatewayDAO.class);
			gatewayDao.updateProductCategoryOrders();
		}
		return results;
	}

	public Map deleteCategory(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting category");
		Category category = (Category) getBusinessObjectFactory().createObject(
				"categoryImplementer");
		BeanUtils.copyProperties(category, parameters);
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		dao.setCategory(category);
		dao.deleteCategory(true);
		return null;
	}

	public Map validateCategory(Category category) throws Exception {
		Map results = new HashMap();
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		dao.setCategory(category);
		dao.loadCategoryFromCode();
		if (dao.getCategory() != null) {
			dao.getDaoFactory().getSession().evict(dao.getCategory());
		}
		boolean insertingButExists = category.getCategoryID() == 0
				&& dao.getCategory() != null
				&& dao.getCategory().getCategoryID() > 0;
		boolean updatingButExists = category.getCategoryID() > 0
				&& dao.getCategory() != null
				&& category.getCategoryID() != dao.getCategory()
						.getCategoryID();
		if (insertingButExists || updatingButExists) {
			results.put("resultCode", "DuplicateRecord");
			results.put("duplicateField", "code");
			results.put("existingRecord", dao.getCategory());
		}
		return results;
	}

	public Map sortCategoriesByName(Map parameters) throws Exception {
		Map results = new HashMap();
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		String workWithCategories = (String) parameters
				.get("workWithCategories");
		boolean isContentPageBool = workWithCategories != null
				&& workWithCategories.equals("Content Pages");
		String parentCategory = (String) parameters.get("parentCategory");
		Collection categoryIDs = null;
		if (parentCategory == null) {
			categoryIDs = loadAllCategoriesSortedByName(isContentPageBool);
		} else {
			int parent = Integer.parseInt(parentCategory);
			categoryIDs = loadSubcategoriesSortedByName(isContentPageBool,
					parent);
		}
		int sortOrder = 0;
		Iterator i = categoryIDs.iterator();
		while (i.hasNext()) {
			Integer categoryID = (Integer) i.next();
			sortOrder++;
			if (log.isDebugEnabled())
				log.debug("Updating categoryOrder to " + sortOrder
						+ " where categoryID is " + categoryID);
			dao.updateCategoryOrderByID(categoryID.intValue(), sortOrder);
		}
		results.put("categoriesSorted", new Integer(sortOrder));
		return results;
	}

	public Collection loadAllCategoriesSortedByName(boolean isContentPage)
			throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading all categories sorted by name");
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadAllCategoriesSortedByName(isContentPage);
	}

	public Collection loadSubcategoriesSortedByName(boolean isContentPage,
			int parentCategory) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading subcategories sorted by name");
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadSubcategoriesSortedByName(isContentPage,
				parentCategory);
	}

	public Map sortCategoriesByCode(Map parameters) throws Exception {
		Map results = new HashMap();
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		String workWithCategories = (String) parameters
				.get("workWithCategories");
		boolean isContentPageBool = workWithCategories != null
				&& workWithCategories.equals("Content Pages");
		String parentCategory = (String) parameters.get("parentCategory");
		Collection categoryIDs = null;
		if (parentCategory == null) {
			categoryIDs = loadAllCategoriesSortedByCode(isContentPageBool);
		} else {
			int parent = Integer.parseInt(parentCategory);
			categoryIDs = loadSubcategoriesSortedByCode(isContentPageBool,
					parent);
		}
		int sortOrder = 0;
		Iterator i = categoryIDs.iterator();
		while (i.hasNext()) {
			Integer categoryID = (Integer) i.next();
			sortOrder++;
			if (log.isDebugEnabled())
				log.debug("Updating categoryOrder to " + sortOrder
						+ " where categoryID is " + categoryID);
			dao.updateCategoryOrderByID(categoryID.intValue(), sortOrder);
		}
		results.put("categoriesSorted", new Integer(sortOrder));
		return results;
	}

	public Collection loadAllCategoriesSortedByCode(boolean isContentPage)
			throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading all categories sorted by code");
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadAllCategoriesSortedByCode(isContentPage);
	}

	public Collection loadSubcategoriesSortedByCode(boolean isContentPage,
			int parentCategory) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading subcategories sorted by code");
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadSubcategoriesSortedByCode(isContentPage,
				parentCategory);
	}

	public Collection loadSettings(Map parameters) throws Exception {
		if (log.isDebugEnabled())
			log.debug("About to load category settings.");
		Category category = (Category) getBusinessObjectFactory().createObject(
				"categoryImplementer");
		BeanUtils.copyProperties(category, parameters);
		CategoryDAO dao = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		dao.setCategory(category);
		return dao.loadSettings();
	}

	public Collection processSettings(Map parameters) throws Exception {
		if (log.isDebugEnabled())
			log.debug("About to process category settings.");
		// Load the settings
		Category category = (Category) getBusinessObjectFactory().createObject(
				"categoryImplementer");
		BeanUtils.copyProperties(category, parameters);
		CategoryDAO categoryDAO = (CategoryDAO) getDaoFactory().createDAO(
				"categoryDAOImplementer");
		categoryDAO.setCategory(category);
		Collection settings = categoryDAO.loadSettings();
		parameters.put("settings", settings);
		parameters.put("categoryDAO", categoryDAO);
		if (log.isDebugEnabled())
			log.debug("Determining new category settings to insert.");
		// Create a collection of category settings that are new and must be
		// inserted.
		Collection categorySettingsToInsert = new ArrayList();
		String[] codes = (String[]) parameters.get("code");
		String[] names = (String[]) parameters.get("name");
		String[] descriptions = (String[]) parameters.get("description");
		String[] values = (String[]) parameters.get("value");
		String[] valueTypes = (String[]) parameters.get("valueType");
		for (int i = 0; i < codes.length; i++) {
			boolean insert = true;
			Iterator j = settings.iterator();
			while (j.hasNext()) {
				CategorySetting setting = (CategorySetting) j.next();
				if (setting.getCode().equals(codes[i])) {
					if (log.isDebugEnabled())
						log.debug("Existing category setting: " + codes[i]);
					setting.setCategory(category);
					insert = false;
				}
			}
			if (insert) {
				if (log.isDebugEnabled())
					log.debug("New category setting: " + codes[i]);
				CategorySetting categorySetting =  getBusinessObjectFactory()
						.createObject(CategorySetting.class);
				categorySetting.setCategoryID(Integer
						.parseInt((String) parameters.get("categoryID")));
				categorySetting.setCode(codes[i]);
				categorySetting.setName(names[i]);
				categorySetting.setDescription(descriptions[i]);
				categorySetting.setValueType(valueTypes[i]);
				categorySetting.setType((String) parameters.get("type"));
				if (valueTypes[i].equals("integerValue")
						|| valueTypes[i].equals("booleanValue")) {
					categorySetting
							.setIntegerValue(Integer.parseInt(values[i]));
				} else if (valueTypes[i].equals("smallValue")) {
					categorySetting.setSmallValue(values[i]);
				} else if (valueTypes[i].equals("mediumValue")) {
					categorySetting.setMediumValue(values[i]);
				} else if (valueTypes[i].equals("textValue")) {
					categorySetting.setTextValue(values[i]);
				}
				categorySetting.setCategory(category);
				categorySettingsToInsert.add(categorySetting);
			}
		}
		parameters.put("categorySettingsToInsert", categorySettingsToInsert);
		// Process the updates and inserts in the database
		ProductGatewayDAO dao = (ProductGatewayDAO) getDaoFactory().createDAO(
				"productGatewayDAOImplementer");
		return dao.processCategorySettings(true, parameters);
	}

	public int loadMaxCategoryID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading max category ID.");
		}
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadMaxCategoryID();
	}

	public Collection loadCategoriesForIndexing(int firstCategoryID,
			int lastCategoryID) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading categories for indexing.");
			log.info("firstCategoryID: " + firstCategoryID);
			log.info("lastCategoryID: " + lastCategoryID);
		}
		CategoryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CategoryGatewayDAO.class);
		return gatewayDAO.loadCategoriesForIndexing(firstCategoryID,
				lastCategoryID);
	}

}
