package com.softslate.commerce.customer.core;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.Settings;

/**
 * Provides miscellaneous static methods used by various classes in the Struts
 * layer.
 * 
 * @author David Tobey
 */
public class AppUtils {

	static Log log = LogFactory.getLog(AppUtils.class);

	/**
	 * The name of the directory under the "/WEB-INF/layouts" directory in the
	 * application where the system's default JSP templates reside. (See the
	 * "Creating a Custom Layout" chapter of the User Guide.)
	 */
	public static final String systemLayoutDirectory = "default";

	/**
	 * Load properties from a path to a .properties file in the form of
	 * "some/package/settings.properties" represented by the argument.
	 * 
	 * @param path
	 *            The path to a .properties file under the application's
	 *            classpath.
	 * @param servletContext
	 *            Used to locate the .properties file through its
	 *            <code>getRealPath</code> method.
	 * @return A <code>Properties</code> object represented by the .properties
	 *         file.
	 */
	public static Properties loadProperties(String path, ServletContext servletContext) {
		String pathToProperties = servletContext.getRealPath("/WEB-INF/classes/" + path);
		File propertiesFile = new File(pathToProperties);
		return loadProperties(propertiesFile);
	}

	/**
	 * Load properties from a given <code>File</code> object.
	 * 
	 * @param file
	 *            The <code>File</code> object.
	 * @return The properties in the given <code>File</code>.
	 */
	public static Properties loadProperties(File file) {
		Properties properties = null;
		FileInputStream fis = null;
		BufferedInputStream bis = null;
		try {
			fis = new FileInputStream(file);
			bis = new BufferedInputStream(fis);
			properties = new Properties();
			properties.load(bis);
			bis.close();
			fis.close();
		} catch (IOException e) {
			log.error("Exception in loading properties", e);
			properties = null;
		} finally {
			fis = null;
			bis = null;
		}
		return properties;
	}

	/**
	 * Load properties from a given <code>String</code> represent the path to
	 * the properties file..
	 * 
	 * @param path
	 *            The path to the properties file.
	 * @return The properties in the file.
	 */
	public static Properties loadProperties(String path, Properties defaults) {
		InputStream is;
		BufferedInputStream bis = null;
		try {
			Properties properties = null;
			if (defaults == null) {
				properties = new Properties();
			} else {
				properties = new Properties(defaults);
			}
			is = AppUtils.class.getResourceAsStream(path);
			bis = new BufferedInputStream(is);
			properties.load(bis);
			bis.close();
			is.close();
			return properties;
		} catch (IOException e) {
			log.error("Exception in loading properties", e);
			return null;
		} finally {
			is = null;
			bis = null;
		}
	}

	/**
	 * Replaces strings matching the {@link #systemLayoutDirectory} with the
	 * user's custom layout directory. If the resulting path doesn't map to an
	 * existing file, returns the original string. Used by
	 * {@link com.softslate.commerce.customer.core.BaseRequestProcessor}<!-- -->
	 * and {@link com.softslate.commerce.customer.core.AppInsertTag} to handle
	 * forwards coming from the Struts request. (See the "Creating a Custom
	 * Layout" chapter of the User Guide.)
	 * 
	 * @param uri
	 *            The original uri, containing the
	 *            {@link #systemLayoutDirectory}.
	 * @param layout
	 *            The user's custom layout directory.
	 * @param servletContext
	 *            Used to determine if the resulting path maps to an existing
	 *            file, through the <code>getRealPath</code> method.
	 * @return A new uri mapping to the custom path to a file under the user's
	 *         custom directory, or the original uri, if the new one does not
	 *         exist.
	 */
	public static String alterURI(String uri, String layout, ServletContext servletContext) {
		String newUri = null;
		newUri = findLayoutURIIfFileExists(uri, layout, servletContext);
		Settings settings = (Settings) servletContext.getAttribute("settings");
		if (settings != null) {
			if (newUri == null) {
				newUri = findLayoutURIIfFileExists(uri, (String) settings.getValue("defaultLayout"), servletContext);
			}
			if (newUri == null) {
				newUri = findLayoutURIIfFileExists(uri, (String) settings.getValue("commonLayout"), servletContext);
			}
			if (newUri == null) {
				newUri = findLayoutURIIfFileExists(uri, (String) settings.getValue("baseLayout"), servletContext);
			}
		}
		if (newUri == null) {
			if (log.isDebugEnabled())
				log.debug("Neither layout file, common file, nor base file exist; including: " + uri);
			newUri = uri;
		}
		return newUri;
	}

	public static String findLayoutURIIfFileExists(String uri, String layout, ServletContext servletContext) {
		if (layout != null && layout.length() > 0 && !layout.equals(systemLayoutDirectory)) {
			String newUri = replaceLayout(uri, layout);
			String newPath = servletContext.getRealPath(newUri);
			File newFile = new File(newPath);
			if (newFile.exists()) {
				if (log.isDebugEnabled())
					log.debug("Including user's layout file: " + newPath);
				return newUri;
			}
		}
		return null;
	}

	public static String replaceLayout(String uri, String layout) {
		Pattern pattern = Pattern.compile("/" + systemLayoutDirectory + ".*?/");
		String oldUri = uri;
		Matcher matcher = pattern.matcher(oldUri);
		// Replace systemLayoutDirectory with the current layout
		String newUri = matcher.replaceFirst("/" + layout + "/");
		return newUri;
	}

}
