package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to categories.
 * 
 * @author David Tobey
 */
public class CategoryGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate
		implements CategoryGatewayDAO {

	static Log log = LogFactory.getLog(CategoryGatewayDAOHibernate.class);

	public Map loadCategoriesAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteCategories(Map parameters, BusinessObjectFactory bof)
			throws Exception {
		Map result = editDeleteItems(parameters, bof);
		evictCache();
		return result;
	}

	public Collection loadAllCategories() throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectAll");
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return results;
	}

	public Collection loadAllCategoriesSortedByName(boolean isContentPage)
			throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectAllSortedByName");
			q.setBoolean("isContentPage", isContentPage);
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return results;
	}

	public Collection loadSubcategoriesSortedByName(boolean isContentPage,
			int parentCategory) throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"category.selectSubcategoriesSortedByName");
			q.setBoolean("isContentPage", isContentPage);
			q.setInteger("parentCategory", parentCategory);
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return results;
	}

	public Collection loadAllCategoriesSortedByCode(boolean isContentPage)
			throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectAllSortedByCode");
			q.setBoolean("isContentPage", isContentPage);
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return results;
	}

	public Collection loadSubcategoriesSortedByCode(boolean isContentPage,
			int parentCategory) throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"category.selectSubcategoriesSortedByCode");
			q.setBoolean("isContentPage", isContentPage);
			q.setInteger("parentCategory", parentCategory);
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return results;
	}

	public int loadMaxCategoryID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.maxCategoryID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadCategoriesForIndexing(int firstCategoryID,
			int lastCategoryID) throws DataAccessException {
		Collection categories = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectForIndexing");
			q.setInteger("firstCategoryID", firstCategoryID);
			q.setInteger("lastCategoryID", lastCategoryID);
			categories = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return categories;
	}

	public void updateNullifyParentIDs(String[] parentCategoryIDs)
			throws Exception {
		Integer[] integerIDs = new Integer[parentCategoryIDs.length];
		for (int i = 0; i < parentCategoryIDs.length; i++) {
			integerIDs[i] = new Integer(parentCategoryIDs[i]);
		}
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.updateNullifyParentIDs");
			q.setParameterList("parentCategoryIDs", integerIDs);
			q.executeUpdate();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return;
	}

	public Collection loadAllCategoriesInRange(int firstID, int lastID)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading categories within an id range: " + firstID + "-"
					+ lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}

	public Criteria processCommonCriteria(Criteria crit, Map parameters) {
		String workWithCategories = (String) parameters
				.get("workWithCategories");
		if (workWithCategories == null) {
			crit = super.processCommonCriteria(crit, parameters);
		} else if (workWithCategories.equals("Categories")) {
			crit = super.processCommonCriteria(crit, parameters);
			crit.add(Restrictions.eq("isContentPage", new Boolean(false)));
		} else if (workWithCategories.equals("Content Pages")) {
			crit = super.processCommonCriteria(crit, parameters);
			crit.add(Restrictions.eq("isContentPage", new Boolean(true)));
		} else {
			crit = super.processCommonCriteria(crit, parameters);
		}
		return crit;
	}
}
